@@ -10,6 +10,7 @@ $CPAN::Config = {
                  'build_dir' => qq[$cwd/t/dot-cpan/build],
                  #'bzip2' => q[/bin/bzip2],
                  'cache_metadata' => q[1],
+                 colorize_output=>0,
                  'cpan_home' => qq[$cwd/t/dot-cpan],
                  #'curl' => q[],
                  #'ftp' => q[],
@@ -43,6 +44,7 @@ $CPAN::Config = {
                  'show_upload_date' => q[0],
                  #'tar' => q[/bin/tar],
                  'term_is_latin' => q[0],
+                 'term_ornaments' => q[0],
                  #'unzip' => q[/usr/bin/unzip],
                  'urllist' => [qq[file://$cwd/t/CPAN]],
                  'wait_list' => [q[wait://ls6.informatik.uni-dortmund.de:1404]],
